package nu.dll.md5integrity;

import org.eclipse.swt.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.browser.*;
import org.eclipse.swt.graphics.*;
import org.eclipse.swt.program.Program;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;

public class GUI { // implements ProgressListener, CheckActionCallback {
    Display display;
    Shell shell;
    Text dirText;
    Label statusLabel;
    Button scanBtn;
    Button checkBtn;

    public GUI() {
	display = new Display();
	setupShell();
	setupWidgets();
    }

    void setupShell() {
	shell = new Shell(display, SWT.BORDER | SWT.RESIZE | SWT.CLOSE);
	shell.setText("MD5i GUI");
	GridLayout l = new GridLayout();
	l.numColumns = 2;
	shell.setLayout(l);	
    }

    void setupWidgets() {
	dirText = new Text(shell, SWT.BORDER);
	GridData gd = new GridData(GridData.FILL_HORIZONTAL);
	gd.widthHint = 200;
	dirText.setLayoutData(gd);
	Button dirButton = new Button(shell, SWT.BORDER);
	dirButton.setText("Browse...");
	dirButton.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
		    DirectoryDialog dialog = new DirectoryDialog(shell);
		    dialog.setMessage("Choose folder");
		    String selected = dialog.open();
		    if (selected != null) {
			dirText.setText(selected);
		    }
		}
	    });

	dirText.addModifyListener(new ModifyListener() {
		public void modifyText(ModifyEvent e) {
		    File f = new File(dirText.getText());
		    if (f.exists() && f.isDirectory()) {
			setStatus("Click a button to start");
			enableButtons(true);
		    } else {
			setStatus("Not found: choose a directory");
			enableButtons(false);
		    }
		}
	    });

	statusLabel = new Label(shell, SWT.NONE);
	gd = new GridData(GridData.FILL_HORIZONTAL);
	gd.horizontalSpan = 2;
	statusLabel.setLayoutData(gd);
	statusLabel.setText("Choose a directory to check");

	Composite btnComp = new Composite(shell, SWT.NONE);
	btnComp.setLayout(new FillLayout());
	gd = new GridData(GridData.FILL_HORIZONTAL);
	btnComp.setLayoutData(gd);

	scanBtn = new Button(btnComp, SWT.NONE);
	scanBtn.setText("Build database");
	scanBtn.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
		    scan();
		}
	    });
	scanBtn.setEnabled(false);
	checkBtn = new Button(btnComp, SWT.NONE);
	checkBtn.setText("Check");
	checkBtn.addSelectionListener(new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
		    check();
		}
	    });
	checkBtn.setEnabled(false);

	shell.pack();
    }

    void enableButtons(final boolean enabled) {
	display.syncExec(new Runnable() {
		public void run() {
		    checkBtn.setEnabled(enabled);
		    scanBtn.setEnabled(enabled);
		}
	    });	
    }

    void setStatus(final String s) {
	display.asyncExec(new Runnable() {
		public void run() {
		    statusLabel.setText(s);
		}
	    });
    }

    void scan() {
	final File root = new File(dirText.getText());
	enableButtons(false);
	Thread t = new Thread() {
		public void run() {
		    try {
			setStatus("Scanning...");
			MD5Integrity md5i = new MD5Integrity(true);
			md5i.setProgressListener(new ProgressAdapter() {
				public void fileChecking(File f) {
				    setStatus("Scanning: " + f.getName());
				}
			    });
			md5i.initScan(true);
			md5i.scan(root);
			md5i.sync();
			setStatus("Scan complete.");
		    } catch (IOException ex1) {
			setStatus("Error: " + ex1.getMessage());
			ex1.printStackTrace();
		    } finally {
			enableButtons(true);
		    }
		}
	    };
	t.setDaemon(true);
	t.start();
    }

    void check() {
	final File root = new File(dirText.getText());
	enableButtons(false);
	Thread t = new Thread() {
		public void run() {
		    try {
			setStatus("Checking...");
			MD5Integrity md5i = new MD5Integrity(true);
			md5i.setProgressListener(new ProgressAdapter() {
				public void fileChecking(File f) {
				    setStatus("Checking: " + f.getName());
				}
			    });
			md5i.setCheckActionCallback(new CheckActionCallback() {
				public int fileModified(FileData f1, FileData f2) {
				    int v = queryFileModified(f1, f2);
				    if (v == CheckActionCallback.ACTION_ABORT) {
					setStatus("Aborted.");
				    }
				    return v;
				}
			    });
			try {
			    md5i.initCheck();
			} catch (FileNotFoundException ex1) {
			    display.asyncExec(new Runnable() {
				    public void run() {
					MessageBox mb = new MessageBox(shell, SWT.ICON_ERROR | SWT.OK);
					mb.setText("Database not found");
					mb.setMessage("The database file could not be found. " + 
						      "Use \"Build database\" first to create " +
						      "your MD5 database.");
					mb.open();
				    }
				});
			    setStatus("Scan aborted.");
			    return;
			}
			md5i.check(root);
			md5i.sync();
			int filecount = md5i.getFailedCheckCount();
			if (filecount == 0) {
			    setStatus("Check completed: all files OK.");
			} else {
			    setStatus("Check completed: " + filecount + " files modified.");
			}
		    } catch (IOException ex1) {
			setStatus("Error: " + ex1.getMessage());
			ex1.printStackTrace();
		    } finally {
			enableButtons(true);
		    }
		}
	    };
	t.setDaemon(true);
	t.start();
    }

    static class MutableInt {
	int value;
	MutableInt(int value) { this.value = value; }
    }

    int queryFileModified(final FileData f1, final FileData f2) {
	final MutableInt rv = new MutableInt(0);
	display.syncExec(new Runnable() {
		public void run() {
		    MessageBox mb = new MessageBox(shell, SWT.ICON_INFORMATION |
						   SWT.ABORT | SWT.RETRY | SWT.IGNORE);
		    if (f1 == null) {
			mb.setText("New file");
			mb.setMessage("The file " + f2.canonicalPath + " was not found " +
				      "in the MD5 database. Do you want to add it " +
				      "(retry), ignore it (ignore) or abort the scan " +
				      "(abort)?");
		    } else {
			mb.setText("File contents changed");
			mb.setMessage("File: " + f1.canonicalPath + "\n" + 
				      "Old MD5: " + hexstring(f1.md5) + "\n" + 
				      "New MD5: " + hexstring(f2.md5) + "\n" + 
				      "Old timestamp: " + new Date(f1.lastModified) + "\n" + 
				      "New timestamp: " + new Date(f2.lastModified) + "\n" + 
				      "\n" + 
				      "Do you want to update the database with the new " +
				      "information (retry), ignore it (ignore) or abort the scan " +
				      "(abort)?");
		    }
		    int value = mb.open();
		    if (value == SWT.ABORT) {
			rv.value = CheckActionCallback.ACTION_ABORT;
		    } else if (value == SWT.RETRY) {
			rv.value = CheckActionCallback.ACTION_UPDATEDB;
		    } else {
			rv.value = CheckActionCallback.ACTION_IGNORE;
		    }
		    
		}
	    });
	return rv.value;
    }

    static String hexstring(byte[] b) {
	return MD5Integrity.hexstring(b);
    }
    
    public void show() {
	shell.open();
    }

    public void run() {
	while (!shell.isDisposed()) {
	    if (!display.readAndDispatch()) {
		display.sleep();
	    }
	}
    }

    public static void main(String[] argv) {
	GUI gui = new GUI();
	gui.show();
	gui.run();
    }
}
